/hg/icedtea6: - Support for Chromium and Firefox 3.6 A1

dbhole at icedtea.classpath.org dbhole at icedtea.classpath.org
Wed Aug 26 12:02:46 PDT 2009


changeset 8353f4e67ec1 in /hg/icedtea6
details: http://icedtea.classpath.org/hg/icedtea6?cmd=changeset;node=8353f4e67ec1
author: Deepak Bhole <dbhole at redhat.com>
date: Wed Aug 26 15:06:47 2009 -0400

	- Support for Chromium and Firefox 3.6 A1
	- Added object construction support
	- Port over cookie/cert cn mismatch/etc. fixes from current plugin


diffstat:

18 files changed, 869 insertions(+), 385 deletions(-)
ChangeLog                                                         |   81 +++
plugin/icedteanp/IcedTeaJavaRequestProcessor.cc                   |   54 +-
plugin/icedteanp/IcedTeaJavaRequestProcessor.h                    |    9 
plugin/icedteanp/IcedTeaNPPlugin.cc                               |  269 +++++++---
plugin/icedteanp/IcedTeaNPPlugin.h                                |   11 
plugin/icedteanp/IcedTeaPluginRequestProcessor.cc                 |  250 ++++-----
plugin/icedteanp/IcedTeaPluginRequestProcessor.h                  |   23 
plugin/icedteanp/IcedTeaPluginUtils.cc                            |   28 -
plugin/icedteanp/IcedTeaPluginUtils.h                             |    8 
plugin/icedteanp/IcedTeaScriptablePluginObject.cc                 |   53 +
plugin/icedteanp/IcedTeaScriptablePluginObject.h                  |    5 
plugin/icedteanp/java/sun/applet/MethodOverloadResolver.java      |  134 ++++
plugin/icedteanp/java/sun/applet/PluginAppletSecurityContext.java |  108 +++-
plugin/icedteanp/java/sun/applet/PluginAppletViewer.java          |    3 
plugin/icedteanp/java/sun/applet/PluginCookieInfoRequest.java     |   53 -
plugin/icedteanp/java/sun/applet/PluginCookieManager.java         |   88 +++
plugin/icedteanp/java/sun/applet/PluginCookieStore.java           |   73 --
plugin/icedteanp/java/sun/applet/PluginMain.java                  |    4 

diffs (truncated from 2067 to 500 lines):

diff -r f1cd6ab69a53 -r 8353f4e67ec1 ChangeLog
--- a/ChangeLog	Wed Aug 26 01:14:21 2009 -0400
+++ b/ChangeLog	Wed Aug 26 15:06:47 2009 -0400
@@ -1,3 +1,84 @@ 2009-08-26  Deepak Bhole <dbhole at redhat.
+2009-08-26  Deepak Bhole <dbhole at redhat.com>
+
+	* plugin/icedteanp/IcedTeaJavaRequestProcessor.cc
+	(createJavaObjectFromVariant): Add conditional compilation blocks to
+	handle new API along with old. Call renamed newObjectWithConstructor
+	function.
+	(newObject): New function. Creates an object based on the most optimum
+	cost with arbitrary arguments.
+	(newObjectWithConstructor): Renamed from newObject(). Also, take source as
+	an argument to provide to Java.
+	* plugin/icedteanp/IcedTeaJavaRequestProcessor.h: Change as per above .cc
+	file changes.
+	* plugin/icedteanp/IcedTeaNPPlugin.cc: Include XPCOM specific headers only
+	if xulrunner version is < 1.9.2. Instantiate internal Mozilla objects only
+	if xulrunner < 1.9.2.
+	(GCJ_New): Correctly account for documentbase length when allocating tag
+	memory to prevent corruption.
+	(GCJ_Destroy): Remove old instance mappings when destroyed.
+	(GCJ_GetJavaClass): Remove unused method.
+	(get_cookie_info): Change object signature to include length result. Based
+	on xulrunner version, either use internal Mozilla API for fetching cookie
+	info, or use NPN_GetValueForURL.
+	(plugin_get_documentbase): Define for when xulrunner is >= 1.9.2.
+	(consume_message): Deal with single proxy URL rather than components.
+	Update to handle when xulrunner is >= 1.9.2
+	(get_proxy_info): Change signature to deal with single url rather than
+	components. Use NPN_GetValueForURL if xulrunner is >= 1.9.2.
+	(NP_Initialize): Add pointers for getvalueforurl and setvalueforurl in the
+	NPPluginFuncs table. Also, handle case for when browser provides the
+	plugin .so name as the link location rather than target location.
+	* plugin/icedteanp/IcedTeaNPPlugin.h: Include XPCOM specific headers only
+	if xulrunner version is < 1.9.2.
+	* plugin/icedteanp/IcedTeaPluginRequestProcessor.cc
+	(eval): Use the new NPN_PluginThreadAsynCcall function to dispatch events
+	to main thread asynchronously.
+	(call): Same.
+	(setMember): Same.
+	(getMember): Same.
+	(storeVariantInJava): Handle changed API when xulrunner >= 1.9.2
+	(_setMember): Account for changes in the way that the callerpacks
+	arguments.
+	(_getMember): Same.
+	(_call): Same.
+	(_eval): Same.
+	* plugin/icedteanp/IcedTeaPluginRequestProcessor.h: Include Mozilla
+	specific headers only if xulrunner is < 1.9.2. Remove old ThreadData
+	struct and add new AyncCallThreadData struct that contains results and
+	arguments.
+	* plugin/icedteanp/IcedTeaPluginUtils.cc
+	(getSourceFromInstance): Provide null source location due to lack of a
+	secure way to get documentbase.
+	(printNPVariant): Account for change API in xulrunner 1.9.2.
+	(NPVariantToString): Same.
+	* plugin/icedteanp/IcedTeaPluginUtils.h: Include Mozilla specific headers 
+	only if xulrunner is < 1.9.2.
+	* plugin/icedteanp/IcedTeaScriptablePluginObject.cc
+	(invoke): Pass exception over to JS engine.
+	(construct): Implement object construction.
+	* plugin/icedteanp/IcedTeaScriptablePluginObject.h: Include Mozilla
+	specific headers only if xulrunner is < 1.9.2.
+	* plugin/icedteanp/java/sun/applet/MethodOverloadResolver.java
+	(getMatchingConstructor): New method, returns a matching constructor based
+	on arguments.
+	(getMatchingConstructors): New helper method for getMatchingConstructor.
+	* plugin/icedteanp/java/sun/applet/PluginAppletSecurityContext.java
+	(handleMessage): Handle new type of call, "NewObjectWithConstructor".
+	(checkPermission): Disable checks for the time being, as NPRuntime does
+	not allow cross-site calling anyway.
+	* plugin/icedteanp/java/sun/applet/PluginAppletViewer.java
+	(setStatus): Remove newlines from status messages.
+	* plugin/icedteanp/sun/applet/PluginCookieInfoRequest.java
+	(parseReturn): Store cookie info as a string rather than a list of
+	HttpCookie objects.
+	(getObject): Return the new cookie string.
+	* plugin/icedteanp/sun/applet/PluginCookieManager.java: New file, extends
+	CookieManager as is set as the default cookie manager for the plugin.
+	* plugin/icedteanp/sun/applet/PluginCookieStore.java : Deleted. New design
+	uses a custom cookiemanager rather than just a custom store.
+	* plugin/icedteanp/sun/applet/PluginMain.java
+	(init): Wire in the new cookie manager.
+
 2009-08-26  Deepak Bhole <dbhole at redhat.com>
 
 	* plugin/icedtea/sun/applet/PluginAppletViewer.java
diff -r f1cd6ab69a53 -r 8353f4e67ec1 plugin/icedteanp/IcedTeaJavaRequestProcessor.cc
--- a/plugin/icedteanp/IcedTeaJavaRequestProcessor.cc	Wed Aug 26 01:14:21 2009 -0400
+++ b/plugin/icedteanp/IcedTeaJavaRequestProcessor.cc	Wed Aug 26 15:06:47 2009 -0400
@@ -122,7 +122,6 @@ JavaRequestProcessor::newMessageOnBus(co
 			{
 			    result_ready = true; // nothing else to do
 			} else if ((message_parts->at(4) == "CallMethod") ||
-
 					   (message_parts->at(4) == "CallStaticMethod"))
 			{
 
@@ -656,8 +655,11 @@ JavaRequestProcessor::createJavaObjectFr
     else if (NPVARIANT_IS_STRING(variant))
     {
     	className = "java.lang.String";
-
+#if MOZILLA_VERSION_COLLAPSED < 1090200
     	stringArg += NPVARIANT_TO_STRING(variant).utf8characters;
+#else
+    	stringArg += NPVARIANT_TO_STRING(variant).UTF8Characters;
+#endif
     } else {
     	alreadyCreated = true;
     }
@@ -703,7 +705,7 @@ JavaRequestProcessor::createJavaObjectFr
 		std::string arg = std::string();
 		arg.append(*(java_result->return_string));
 		args.push_back(arg);
-		java_result = java_request.newObject("[System]", jsObjectClassID, jsObjectConstructorID, args);
+		java_result = java_request.newObjectWithConstructor("[System]", jsObjectClassID, jsObjectConstructorID, args);
 
         if (java_result->error_occurred) {
             printf("Unable to create requested object\n");
@@ -808,7 +810,46 @@ JavaRequestProcessor::getObjectClass(std
 }
 
 JavaResultData*
-JavaRequestProcessor::newObject(std::string source, std::string objectID,
+JavaRequestProcessor::newObject(std::string source, std::string classID,
+                                const NPVariant* args, int argCount)
+{
+    JavaRequestProcessor* java_request;
+    std::string message = std::string();
+
+    this->instance = 0; // context is always 0 (needed for java-side backwards compat.)
+    this->reference = IcedTeaPluginUtilities::getReference();
+
+    IcedTeaPluginUtilities::constructMessagePrefix(0, reference, source, &message);
+    message += " NewObject ";
+    message += classID;
+    message += " ";
+
+    // First, we need to load the arguments into the java-side table
+    for (int i=0; i < argCount; i++) {
+        int objectID = createJavaObjectFromVariant(args[i]);
+        if (objectID == 0)
+        {
+            result->error_occurred = true;
+            result->error_msg->append("Unable to create arguments");
+            return result;
+        }
+
+        char* id = (char*) malloc(sizeof(char)*32);
+        sprintf(id, "%d", objectID);
+        message += id;
+        message += " ";
+        free(id);
+    }
+
+    postAndWaitForResponse(message);
+
+    IcedTeaPluginUtilities::releaseReference();
+
+    return result;
+}
+
+JavaResultData*
+JavaRequestProcessor::newObjectWithConstructor(std::string source, std::string classID,
                                 std::string methodID,
                                 std::vector<std::string> args)
 {
@@ -819,8 +860,8 @@ JavaRequestProcessor::newObject(std::str
 	this->reference = IcedTeaPluginUtilities::getReference();
 
 	IcedTeaPluginUtilities::constructMessagePrefix(0, reference, source, &message);
-	message += " NewObject ";
-	message += objectID;
+	message += " NewObjectWithConstructor ";
+	message += classID;
 	message += " ";
 	message += methodID;
 	message += " ";
@@ -980,3 +1021,4 @@ JavaRequestProcessor::getAppletObjectIns
 
     return result;
 }
+
diff -r f1cd6ab69a53 -r 8353f4e67ec1 plugin/icedteanp/IcedTeaNPPlugin.cc
--- a/plugin/icedteanp/IcedTeaNPPlugin.cc	Wed Aug 26 01:14:21 2009 -0400
+++ b/plugin/icedteanp/IcedTeaNPPlugin.cc	Wed Aug 26 15:06:47 2009 -0400
@@ -47,12 +47,15 @@ exception statement from your version. *
 #include <sys/types.h>
 #include <unistd.h>
 
+#if MOZILLA_VERSION_COLLAPSED < 1090200
 // Documentbase retrieval includes.
 #include <nsIPluginInstance.h>
 #include <nsIPluginInstancePeer.h>
 #include <nsIPluginTagInfo2.h>
+#endif
 
 // API's into Mozilla
+#if MOZILLA_VERSION_COLLAPSED < 1090200
 #include <nsCOMPtr.h>
 #include <nsICookieService.h>
 #include <nsIDNSRecord.h>
@@ -66,9 +69,10 @@ exception statement from your version. *
 #include <nsNetCID.h>
 #include <nsStringAPI.h>
 #include <nsServiceManagerUtils.h>
+#endif
 
 // Liveconnect extension
- #include "IcedTeaScriptablePluginObject.h"
+#include "IcedTeaScriptablePluginObject.h"
 #include "IcedTeaNPPlugin.h"
 
 // Error reporting macros.
@@ -132,8 +136,10 @@ exception statement from your version. *
 #define FAILURE_MESSAGE "gcjwebplugin error: Failed to run %s." \
   "  For more detail rerun \"firefox -g\" in a terminal window."
 
+#if MOZILLA_VERSION_COLLAPSED < 1090200
 // Documentbase retrieval required definition.
 static NS_DEFINE_IID (kIPluginTagInfo2IID, NS_IPLUGINTAGINFO2_IID);
+#endif
 
 // Data directory for plugin.
 static gchar* data_directory = NULL;
@@ -180,14 +186,14 @@ PluginRequestProcessor* plugin_req_proc;
 // Sends messages to Java over the bus
 JavaMessageSender* java_req_proc;
 
-GQuark ITNP_PLUGIN_ERROR = g_quark_from_string("IcedTeaNPPlugin");
-
+#if MOZILLA_VERSION_COLLAPSED < 1090200
 // Documentbase retrieval type-punning union.
 typedef union
 {
   void** void_field;
   nsIPluginTagInfo2** info_field;
 } info_union;
+#endif
 
 // Static instance helper functions.
 // Have the browser allocate a new GCJPluginData structure.
@@ -211,8 +217,8 @@ static void plugin_stop_appletviewer ();
 // Uninitialize GCJPluginData structure
 static void plugin_data_destroy (NPP instance);
 
-NS_IMETHODIMP get_cookie_info(const char* siteAddr, char** cookieString);
-void get_proxy_info(const char* siteAddr, char** proxy_scheme, char** proxy_host, char** proxy_port, GError *error);
+NPError get_cookie_info(const char* siteAddr, char** cookieString, uint32_t* len);
+NPError get_proxy_info(const char* siteAddr, char** proxy, uint32_t* len);
 void decode_url(const gchar* url, gchar** decoded_url);
 void consume_message(gchar* message);
 void start_jvm_if_needed();
@@ -336,7 +342,7 @@ GCJ_New (NPMIMEType pluginType, NPP inst
       // Send applet tag message to appletviewer.
       applet_tag = plugin_create_applet_tag (argc, argn, argv);
 
-      tag_message = (gchar*) malloc(strlen(applet_tag)*sizeof(gchar) + 1024);
+      tag_message = (gchar*) malloc(strlen(applet_tag)*sizeof(gchar) + strlen(documentbase)*sizeof(gchar) + 32);
       g_sprintf(tag_message, "instance %d tag %s %s", instance_counter, documentbase, applet_tag);
 
       //plugin_send_message_to_appletviewer (data, data->instance_string);
@@ -636,6 +642,11 @@ GCJ_Destroy (NPP instance, NPSavedData**
       plugin_data_destroy (instance);
     }
 
+  int id = get_id_from_instance(instance);
+
+  g_hash_table_remove(instance_to_id_map, instance);
+  g_hash_table_remove(id_to_instance_map, GINT_TO_POINTER(id));
+
   PLUGIN_DEBUG_0ARG ("GCJ_Destroy return\n");
 
   return NPERR_NO_ERROR;
@@ -844,31 +855,22 @@ GCJ_URLNotify (NPP instance, const char*
   PLUGIN_DEBUG_0ARG ("GCJ_URLNotify return\n");
 }
 
-jref
-GCJ_GetJavaClass (void)
-{
-  PLUGIN_DEBUG_0ARG ("GCJ_GetJavaClass\n");
-
-  PLUGIN_DEBUG_0ARG ("GCJ_GetJavaClass return\n");
-
-  return 0;
-}
-
-NS_IMETHODIMP
-get_cookie_info(const char* siteAddr, char** cookieString)
-{
+NPError
+get_cookie_info(const char* siteAddr, char** cookieString, uint32_t* len)
+{
+#if MOZILLA_VERSION_COLLAPSED < 1090200
   nsresult rv;
   nsCOMPtr<nsIScriptSecurityManager> sec_man =
     do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
 
   if (!sec_man) {
-    return NS_ERROR_FAILURE;
+    return NPERR_GENERIC_ERROR;
   }
 
   nsCOMPtr<nsIIOService> io_svc = do_GetService("@mozilla.org/network/io-service;1", &rv);
 
   if (NS_FAILED(rv) || !io_svc) {
-    return NS_ERROR_FAILURE;
+    return NPERR_GENERIC_ERROR;
   }
 
   nsCOMPtr<nsIURI> uri;
@@ -877,18 +879,44 @@ get_cookie_info(const char* siteAddr, ch
   nsCOMPtr<nsICookieService> cookie_svc = do_GetService("@mozilla.org/cookieService;1", &rv);
 
   if (NS_FAILED(rv) || !cookie_svc) {
-    return NS_ERROR_FAILURE;
+    return NPERR_GENERIC_ERROR;
   }
 
   rv = cookie_svc->GetCookieString(uri, NULL, cookieString);
 
   if (NS_FAILED(rv) || !*cookieString) {
-    return NS_ERROR_FAILURE;
+    return NPERR_GENERIC_ERROR;
   }
 
-  return NS_OK;
-}
-
+#else
+
+  // getvalueforurl needs an NPP instance. Quite frankly, there is no easy way
+  // to know which instance needs the information, as applets on Java side can
+  // be multi-threaded and the thread making a proxy.cookie request cannot be
+  // easily tracked.
+
+  // Fortunately, XULRunner does not care about the instance as long as it is
+  // valid. So we just pick the first valid one and use it. Proxy/Cookie
+  // information is not instance specific anyway, it is URL specific.
+
+  if (browser_functions.getvalueforurl)
+  {
+      GHashTableIter iter;
+      gpointer id, instance;
+
+      g_hash_table_iter_init (&iter, instance_to_id_map);
+      g_hash_table_iter_next (&iter, &instance, &id);
+
+      return browser_functions.getvalueforurl((NPP) instance, NPNURLVCookie, siteAddr, cookieString, len);
+  } else
+  {
+      return NPERR_GENERIC_ERROR;
+  }
+
+#endif
+
+  return NPERR_NO_ERROR;
+}
 
 // HELPER FUNCTIONS
 
@@ -906,6 +934,8 @@ plugin_data_new (GCJPluginData** data)
 
   PLUGIN_DEBUG_0ARG ("plugin_data_new return\n");
 }
+
+
 
 // Documentbase retrieval.  This function gets the current document's
 // documentbase.  This function relies on browser-private data so it
@@ -913,6 +943,7 @@ plugin_data_new (GCJPluginData** data)
 // browser.  We could not find a way to retrieve the documentbase
 // using the original Netscape plugin API so we use the XPCOM API
 // instead.
+#if MOZILLA_VERSION_COLLAPSED < 1090200
 static gchar*
 plugin_get_documentbase (NPP instance)
 {
@@ -970,8 +1001,56 @@ plugin_get_documentbase (NPP instance)
  cleanup_done:
   PLUGIN_DEBUG_0ARG ("plugin_get_documentbase return\n");
 
+  PLUGIN_DEBUG_1ARG("plugin_get_documentbase returning: %s\n", documentbase_copy);
   return documentbase_copy;
 }
+#else
+static gchar*
+plugin_get_documentbase (NPP instance)
+{
+  PLUGIN_DEBUG_0ARG ("plugin_get_documentbase\n");
+
+  char const* documentbase = NULL;
+  gchar* documentbase_copy = NULL;
+
+  // FIXME: This method is not ideal, but there are no known NPAPI call
+  // for this. See thread for more information:
+  // http://www.mail-archive.com/chromium-dev@googlegroups.com/msg04844.html
+
+  // Additionally, since it is insecure, we cannot use it for making
+  // security decisions.
+  NPObject* window;
+  NPString script = NPString();
+  std::string script_str = std::string();
+  NPVariant* location = new NPVariant();
+  std::string location_str = std::string();
+
+  browser_functions.getvalue(instance, NPNVWindowNPObject, &window);
+  script_str += "window.location.href";
+  script.UTF8Characters = script_str.c_str();
+  script.UTF8Length = script_str.size();
+  browser_functions.evaluate(instance, window, &script, location);
+
+  // Strip everything after the last "/"
+  gchar** parts = g_strsplit (NPVARIANT_TO_STRING(*location).UTF8Characters, "/", -1);
+  guint parts_sz = g_strv_length (parts);
+
+  for (int i=0; i < parts_sz - 1; i++)
+  {
+      location_str += parts[i];
+      location_str += "/";
+  }
+
+  documentbase_copy = g_strdup (location_str.c_str());
+
+  // Release references.
+ cleanup_done:
+  PLUGIN_DEBUG_0ARG ("plugin_get_documentbase return\n");
+  PLUGIN_DEBUG_1ARG("plugin_get_documentbase returning: %s\n", documentbase_copy);
+
+  return documentbase_copy;
+}
+#endif
 
 // This function displays an error message if the appletviewer has not
 // been installed correctly.
@@ -1135,21 +1214,22 @@ void consume_message(gchar* message) {
       gchar** parts = g_strsplit (message, " ", 3);
       if (g_str_has_prefix(parts[1], "PluginProxyInfo"))
       {
-        gchar* proxy_scheme = (gchar*) malloc(sizeof(gchar)*32);
-        gchar* proxy_host = (gchar*) malloc(sizeof(gchar)*64);
-        gchar* proxy_port = (gchar*) malloc(sizeof(gchar)*8);
-
-        GError *error = g_error_new(ITNP_PLUGIN_ERROR, 0, "");
+        gchar* proxy;
+        uint32_t len;
 
         gchar* decoded_url = (gchar*) malloc(strlen(parts[2])*sizeof(gchar) + sizeof(gchar));
         decode_url(parts[2], &decoded_url);
-        get_proxy_info(decoded_url, &proxy_scheme, &proxy_host, &proxy_port, error);
 
         gchar* proxy_info;
+
+#if MOZILLA_VERSION_COLLAPSED < 1090200
+	proxy = (char*) malloc(sizeof(char)*2048);
+#endif
+
         proxy_info = g_strconcat ("plugin PluginProxyInfo ", NULL);
-        if (error->code == 0)
+        if (get_proxy_info(decoded_url, &proxy, &len) == NPERR_NO_ERROR)
           {
-            proxy_info = g_strconcat (proxy_info, proxy_scheme, " ", proxy_host, " ", proxy_port, NULL);
+            proxy_info = g_strconcat (proxy_info, proxy, NULL);
           }
 
         PLUGIN_DEBUG_1ARG("Proxy info: %s\n", proxy_info);
@@ -1159,22 +1239,21 @@ void consume_message(gchar* message) {
         decoded_url = NULL;
         g_free(proxy_info);
         proxy_info = NULL;
-        g_free(proxy_scheme);
-        proxy_scheme = NULL;
-        g_free(proxy_host);
-        proxy_host = NULL;
-        g_free(proxy_port);
-        proxy_port = NULL;
+



More information about the distro-pkg-dev mailing list
diff -r f1cd6ab69a53 -r 8353f4e67ec1 plugin/icedteanp/IcedTeaJavaRequestProcessor.h
--- a/plugin/icedteanp/IcedTeaJavaRequestProcessor.h	Wed Aug 26 01:14:21 2009 -0400
+++ b/plugin/icedteanp/IcedTeaJavaRequestProcessor.h	Wed Aug 26 15:06:47 2009 -0400
@@ -171,9 +171,14 @@ class JavaRequestProcessor : BusSubscrib
         /* Returns the class of the given object */
         JavaResultData* getObjectClass(std::string objectID);
 
     	/* Creates a new object */
     	JavaResultData* newObject(std::string source,
-                                  std::string objectID, std::string methodID,
+                                  std::string classID,
+                                  const NPVariant* args, int numArgs);
+
+    	/* Creates a new object when constructor is undetermined */
+    	JavaResultData* newObjectWithConstructor(std::string source, std::string classID,
+                                  std::string methodID,
                                   std::vector<std::string> args);
 
     	/* Returns the class ID */
@@ -171,9 +171,14 @@ class JavaRequestProcessor : BusSubscrib
         JavaResultData* getObjectClass(std::string objectID);
 
-    	/* Creates a new object */
+    	/* Creates a new object with choosable constructor */
     	JavaResultData* newObject(std::string source,